import "dart:io";
import "package:device_info_plus/device_info_plus.dart";
import "package:flutter/foundation.dart";
import "package:flutter_feedback/flutter_feedback.dart";

/// Create issue report according to user input and platform/device
Future<IssueReport> createIssueReport({
  required String appVersion,
  String? issue,
  String? issueExplanation,
  String? userId,
}) async {
  var deviceInfo = DeviceInfoPlugin();

  try {
    if (kIsWeb) {
      var webInfo = await deviceInfo.webBrowserInfo;
      return IssueReport(
        appVersion: appVersion,
        deviceModel: webInfo.platform ?? "Unknown",
        osVersion: webInfo.userAgent ?? "Unknown",
        platform: "Web",
        browserName: webInfo.browserName.name,
        userId: userId,
        issue: issue,
        issueExplanation: issueExplanation,
      );
    } else if (Platform.isAndroid) {
      var androidInfo = await deviceInfo.androidInfo;
      return IssueReport(
        appVersion: appVersion,
        deviceModel: androidInfo.model,
        osVersion: androidInfo.version.release,
        platform: "Android",
        manufacturer: androidInfo.manufacturer,
        userId: userId,
        issue: issue,
        issueExplanation: issueExplanation,
      );
    } else if (Platform.isIOS) {
      var iosInfo = await deviceInfo.iosInfo;
      return IssueReport(
        appVersion: appVersion,
        deviceModel: iosInfo.utsname.machine,
        osVersion: iosInfo.systemVersion,
        platform: "iOS",
        manufacturer: "Apple",
        userId: userId,
        issue: issue,
        issueExplanation: issueExplanation,
      );
    } else if (Platform.isWindows) {
      var windowsInfo = await deviceInfo.windowsInfo;
      return IssueReport(
        appVersion: appVersion,
        deviceModel: windowsInfo.computerName,
        osVersion: windowsInfo.displayVersion,
        platform: "Windows",
        userId: userId,
        issue: issue,
        issueExplanation: issueExplanation,
      );
    } else if (Platform.isMacOS) {
      var macInfo = await deviceInfo.macOsInfo;
      return IssueReport(
        appVersion: appVersion,
        deviceModel: macInfo.model,
        osVersion: macInfo.osRelease,
        platform: "macOS",
        manufacturer: "Apple",
        userId: userId,
        issue: issue,
        issueExplanation: issueExplanation,
      );
    } else if (Platform.isLinux) {
      var linuxInfo = await deviceInfo.linuxInfo;
      return IssueReport(
        appVersion: appVersion,
        deviceModel: linuxInfo.name,
        osVersion: linuxInfo.version ?? "Unknown",
        platform: "Linux",
        userId: userId,
        issue: issue,
        issueExplanation: issueExplanation,
      );
    } else {
      return IssueReport(
        appVersion: appVersion,
        deviceModel: "Unknown",
        osVersion: "Unknown",
        platform: "Unknown",
        userId: userId,
        issue: issue,
        issueExplanation: issueExplanation,
      );
    }
  } on Exception catch (e) {
    debugPrint("Error retrieving device info: $e");
    return IssueReport(
      appVersion: appVersion,
      deviceModel: "Error",
      osVersion: "Error",
      platform: "Unknown",
      userId: userId,
      issue: issue,
      issueExplanation: issueExplanation,
    );
  }
}

///
Future<FeedbackReport> createFeedbackReport({
  required String appVersion,
  required int rating,
  String? feedback,
  String? userId,
}) async {
  var deviceInfo = DeviceInfoPlugin();

  try {
    if (kIsWeb) {
      var webInfo = await deviceInfo.webBrowserInfo;
      return FeedbackReport(
        appVersion: appVersion,
        deviceModel: webInfo.platform ?? "Unknown",
        osVersion: webInfo.userAgent ?? "Unknown",
        platform: "Web",
        browserName: webInfo.browserName.name,
        userId: userId,
        feedback: feedback,
        rating: rating,
      );
    } else if (Platform.isAndroid) {
      var androidInfo = await deviceInfo.androidInfo;
      return FeedbackReport(
        appVersion: appVersion,
        deviceModel: androidInfo.model,
        osVersion: androidInfo.version.release,
        platform: "Android",
        manufacturer: androidInfo.manufacturer,
        userId: userId,
        feedback: feedback,
        rating: rating,
      );
    } else if (Platform.isIOS) {
      var iosInfo = await deviceInfo.iosInfo;
      return FeedbackReport(
        appVersion: appVersion,
        deviceModel: iosInfo.utsname.machine,
        osVersion: iosInfo.systemVersion,
        platform: "iOS",
        manufacturer: "Apple",
        userId: userId,
        feedback: feedback,
        rating: rating,
      );
    } else if (Platform.isWindows) {
      var windowsInfo = await deviceInfo.windowsInfo;
      return FeedbackReport(
        appVersion: appVersion,
        deviceModel: windowsInfo.computerName,
        osVersion: windowsInfo.displayVersion,
        platform: "Windows",
        userId: userId,
        feedback: feedback,
        rating: rating,
      );
    } else if (Platform.isMacOS) {
      var macInfo = await deviceInfo.macOsInfo;
      return FeedbackReport(
        appVersion: appVersion,
        deviceModel: macInfo.model,
        osVersion: macInfo.osRelease,
        platform: "macOS",
        manufacturer: "Apple",
        userId: userId,
        feedback: feedback,
        rating: rating,
      );
    } else if (Platform.isLinux) {
      var linuxInfo = await deviceInfo.linuxInfo;
      return FeedbackReport(
        appVersion: appVersion,
        deviceModel: linuxInfo.name,
        osVersion: linuxInfo.version ?? "Unknown",
        platform: "Linux",
        userId: userId,
        feedback: feedback,
        rating: rating,
      );
    } else {
      return FeedbackReport(
        appVersion: appVersion,
        deviceModel: "Unknown",
        osVersion: "Unknown",
        platform: "Unknown",
        userId: userId,
        feedback: feedback,
        rating: rating,
      );
    }
  } on Exception catch (e) {
    debugPrint("Error retrieving device info: $e");
    return FeedbackReport(
      appVersion: appVersion,
      deviceModel: "Error",
      osVersion: "Error",
      platform: "Unknown",
      userId: userId,
      feedback: feedback,
      rating: rating,
    );
  }
}
